-- Author      : admin
-- Create Date : 10/28/2019 10:19:08 AM

local beql = LibStub("AceAddon-3.0"):GetAddon("beql");
local L = LibStub("AceLocale-3.0"):GetLocale("beql")

local objectiveChanged, objectiveCompleted;
local questAccepted, questAborted, questCompleted, questFailed, questFinished;

function beql:initChat()
    beql:On('QUEST_ACCEPTED', questAccepted);
    beql:On('QUEST_ABORTED', questAborted);
    beql:On('QUEST_COMPLETED', questCompleted);
    beql:On('QUEST_FAILED', questFailed);
    beql:On('QUEST_FINISHED', questFinished);
    beql:On('OBJECTIVE_CHANGED', objectiveChanged);
    beql:On('OBJECTIVE_COMPLETED', objectiveCompleted);
end

--
--- Party Functions
--

--[[----------------------------------------
-- Arguments:
-- string - event - Name of the event
-- number - questID - Id of the quest
-- number - objectiveIndex - index of quests objective
-- number - old - value before update
-- number - new - value after update
--
-- Notes:
-- Handels the event when a objective has changed
--
-- Returns:
-- * nothing
------------------------------------------]]
function objectiveChanged(event, questID, objectiveIndex, old, new)
    if (beql.db.profile.QuestLog.chat.progress ~= true) then
        return
    end

    local text, _type, complete, got, need = beql:GetQuestLogObjective(questID, objectiveIndex);

    if (new < need) then
        beql:Party_Status2Chat(L["Quest objective updated: "] .. text);
    end
end

--[[----------------------------------------
-- Arguments:
-- string - event - Name of the event
-- number - questID - Id of the quest
-- number - objectiveIndex - index of quests objective
--
-- Notes:
-- Handels the event when a objective has completed
--
-- Returns:
-- * nothing
------------------------------------------]]
function objectiveCompleted(event, questID, objectiveIndex)
    if (beql.db.profile.QuestLog.chat.finished ~= true) then
        return;
    end

    local text = beql:GetQuestLogObjective(questID, objectiveIndex);

    beql:Party_Status2Chat(' - ' .. text:gsub(':[^:]*$', '') .. ' ' .. L["Completed!"]);
end

--[[----------------------------------------
-- Arguments:
-- string - event - Name of the event
-- number - questID - Id of the quest
--
-- Notes:
-- Handels the event when a has been aborted
--
-- Returns:
-- * nothing
------------------------------------------]]
function questAborted(event, questID)
    if (beql.db.profile.QuestLog.chat.abandoned ~= true) then
        return;
    end

    local title = beql.questCache[questID];
    beql:Party_Status2Chat(L["Quest abandoned: "] .. title);
end

--[[----------------------------------------
-- Arguments:
-- string - event - Name of the event
-- number - questID - Id of the quest
-- number - objectiveIndex - index of quests objective
--
-- Notes:
-- Handels the event when a objective has completed
--
-- Returns:
-- * nothing
------------------------------------------]]
function questAccepted(event, questID)
    if (beql.db.profile.QuestLog.chat.accept ~= true) then
        return;
    end

    local title = beql:GetQuestLogTitle(questID);
    beql:Party_Status2Chat(L["Quest accepted: "] .. title);
end

--[[----------------------------------------
-- Arguments:
-- string - event - Name of the event
-- number - questID - Id of the quest
--
-- Notes:
-- Handels the event when a quest switched to state completed
--
-- Returns:
-- * nothing
------------------------------------------]]
function questCompleted(event, questID)
    local title = beql:GetQuestLogTitle(questID);

	if (beql.db.profile.QuestLog.InfoOnQuestCompletion) then
		UIErrorsFrame:AddMessage(title .." "..L["Completed!"], 1.0, 0.8, 0.0, 1.0, UIERRORS_HOLD_TIME)

		if (beql.db.profile.QuestLog.InfoSound and beql.db.profile.QuestLog.InfoSound ~= "") then
			PlaySoundFile(beql.db.profile.QuestLog.InfoSound, "Master")
		end
	end

    if (beql.db.profile.QuestLog.chat.finished ~= true) then
        return;
    end

    beql:Party_Status2Chat(title .. ' ' .. L["Completed!"]);
end

--[[----------------------------------------
-- Arguments:
-- string - event - Name of the event
-- number - questID - Id of the quest
--
-- Notes:
-- Handels the event when a quest has failed 
-- and was removed from the questlog
--
-- Returns:
-- * nothing
------------------------------------------]]
function questFailed(event, questID)
    if (beql.db.profile.QuestLog.chat.progress ~= true) then
        return;
    end

    local title = beql.questCache[questID];
    beql:Party_Status2Chat(L["Quest failed: "] .. title);
end

--[[----------------------------------------
-- Arguments:
-- string - event - Name of the event
-- number - questID - Id of the quest
--
-- Notes:
-- Handels the event when a quest has sucessfully complete 
-- and was removed from the questlog
--
-- Returns:
-- * nothing
------------------------------------------]]
function questFinished(event, questID)
    if (beql.db.profile.QuestLog.chat.delivert ~= true) then
        return;
    end

    local title = beql.questCache[questID];
    beql:Party_Status2Chat(L["Quest %d done."]:format(title));
end

--[[----------------------------------------
-- Arguments:
-- string - event - Name of the event
-- number - questID - Id of the quest
-- number - objectiveIndex - index of quests objective
--
-- Notes:
-- Handels the event when a objective has completed
--
-- Returns:
-- * nothing
------------------------------------------]]
function beql:Party_Status2Chat(msg)
    local channel = nil;

	if (beql.db.profile.QuestLog.chat.AnnounceQuest) then
		if (beql.db.profile.QuestLog.chat.chattype == "default") then
			if (UnitInBattleground("player")) then
                channel = 'BATTLEGROUND';
			elseif (IsInRaid()) then
                channel = 'RAID';
			elseif (IsInGroup()) then
                channel = 'PARTY'
			end
		elseif (beql.db.profile.QuestLog.chat.chattype == "group" and IsInGroup()) then
            channel = 'PARTY'
		elseif (beql.db.profile.QuestLog.chat.chattype == "say") then
            channel = 'SAY'
		end
		
        if (channel ~= nil) then
            SendChatMessage(msg, channel);
        end
	end

	if (beql.db.profile.debug and not msg or msg == "") then
		assert(false , "beql:Party_Status2Chat: massage is a '" .. tostring(msg) .. "' value")
	end
end

--[[----------------------------------------
-- Arguments:
-- number - quest id
-- string - quest name
--
-- Notes:
-- Prints the Quest Name/Status and Objectives to the currently opened chatbox
--
-- Returns:
-- * nothing
------------------------------------------]]
function beql:AddQuestStatusToChatFrame(questIndex)
	local chatFrame = DEFAULT_CHAT_FRAME
	local chatType = chatFrame.editBox:GetAttribute("chatType")
	local oText, oType, oNumP, oNumN, oComp
	local text
	if (chatType == "WHISPER") then
		chatType = "SAY"
	end
	local qID, qIndex , qTitle, qLvl, qTag, qObj, qStat, qRec, qdaily, qZone = beqlQ:GetQuestById(questIndex)
	if (qObj and qObj > 0) then
		SendChatMessage("["..qLvl..string.upper(beqlQ:GetShortTagForQuest(qID)).."] "..qTitle, chatType)
		local _, qObjectives = beqlQ:IterateObjectivesForQuest(qID)
		for oText, oItems in pairs(qObjectives) do
			text = ""
			oComp = false;
			oNumP, oNumN, oType = unpack(oItems)
			if (oNumN == nil) then
                oNumN = 1
            end
			if (oNumP == nil) then
                oNumP = 1
            end
			if (oText == nil) then
                oText = "" end
			if (oNumP == oNumN) then
				oComp = true
			end

			text = oText

			if (oComp) then
				text = text.." "..L["(Done)"]
			else
				text = text..": "..oNumP.."/"..oNumN
			end
			if (text and strlen(text) > 0) then
				SendChatMessage(" - "..text, chatType)
			end
		end

        --[[		for i=1, qObj do
        text = ""
        oText, oType, oNumP, oNumN, oComp = beqlQ:GetQuestStatusById(questIndex, i)
        text = oText
        if (finished) then
        text = text.." "..L["(Done)"]
        else
        text = text..": "..oNumP.."/"..oNumN
        end
        if (text and strlen(text) > 0) then
        SendChatMessage(" - "..text, chatType)
        end
		end--]]
	else
		chatFrame:AddMessage(L["No Objectives!"])
	end
	chatFrame.editBox:Hide()
end
